# “La Horde du Contrevent” character analysis
# © 2019-2020 Lunar
# Licensed under WTFPL-2

#install.packages("devtools")
#install.packages("tidyverse")
#install.packages("ggplot2")
#install.packages("ggfittext")
#install.packages("ggthemes")
#devtools::install_github("coolbutuseless/minisvg")
#devtools::install_github("coolbutuseless/devout")
#devtools::install_github("coolbutuseless/devoutsvg")
#devtools::install_github("coolbutuseless/poissoned")

library(tidyverse)
library(grid)
library(ggplot2)
library(ggfittext)
library(ggthemes)
library(devoutsvg)


horde <- read.csv('horde.csv') %>% filter(Partie != "None")

horde %>% group_by(Partie) %>% summarize(Phrases = sum(Phrases))

personnages <- c('Sov',
                 'Pietro',
                 'Oroshi',
                 'Golgoth',
                 'Aoi',
                 'Larco',
                 'Caracole',
                 'Boscavo',
                 'Talweg',
                 'Callirhoe',
                 'Erg',
                 'Coriolis',
                 'Steppe',
                 'Autoursier',
                 'Alme',
                 'Seleme',
                 'Firost',
                 'Learch',
                 'Arval',
                 'Fauconnier',
                 'Barbak',
                 'Dubka'
                 )

personnages_masculins <- c("Golgoth", "Pietro", "Sov","Caracole",
                           "Erg", "Talweg", "Firost", "Autoursier",
                           "Steppe", "Arval", "Fauconnier", "Dubka",
                           "Larco", "Learch", "Boscavo", "Coriolis",
                           "Sveziest", "Barbak")
personnages_feminins <- c("Oroshi", "Alme", "Aoi", "Callirhoe")


patterns <- tribble(
  ~Partie,        ~colour,
  "dialogue",     "#7F7F7F",
  "Alme",         "#FFF000",
  "Aoi",          "#FFF001",
  "Arval",        "#FFF002",
  "Autoursier",   "#FFF003",
  "Barbak",       "#FFF004",
  "Boscavo",      "#FFF005",
  "Callirhoe",    "#FFF006",
  "Caracole",     "#FFF007",
  "Coriolis",     "#FFF008",
  "Dubka",        "#FFF009",
  "Erg",          "#FFF010",
  "Fauconnier",   "#FFF011",
  "Firost",       "#FFF012",
  "Golgoth",      "#FFF013",
  "Larco",        "#FFF014",
  "Learch",       "#FFF015",
  "Oroshi",       "#FFF016",
  "Pietro",       "#FFF017",
  "Sov",          "#FFF018",
  "Steppe",       "#FFF019",
  "Talweg",       "#FFF020",
  "Seleme",       "#FFF021",
)



scale_fill_personnages <-
  scale_fill_manual(
    values = append(set_names(patterns$colour, patterns$Partie),
		    c("Autres" = "#CCCCCC"))
  )

whole_book <- horde %>%
  mutate(Partie = fct_other(Partie, keep = append(personnages, 'dialogue'), other_level = "Autres")) %>%
  group_by(Partie) %>%
  summarize(Signes = sum(Signes),
            Mots = sum(Mots),
            Phrases = sum(Phrases)) %>%
  mutate(Partie = fct_reorder(Partie, Signes)) %>%
  mutate(Partie = fct_relevel(Partie, "Autres"))

whole_book %>%
  gather(key = "Métrique", value="Nombre", factor_key=TRUE, -Partie) %>%
  ggplot(aes(x = Partie, y = Nombre, fill = Partie)) +
    geom_col() +
    scale_fill_personnages +
    geom_bar_text(aes(colour = Partie),
                  formatter = scales::label_number(),
                  place = "right",
                  grow = FALSE,
                  min.size = 12, outside = TRUE) +
    scale_colour_manual(
      values = append(set_names(patterns$colour, patterns$Partie),
                      c("Autres" = "#CCCCCC"))
    ) +
    scale_y_continuous(
      label = scales::label_number(),
      minor_breaks = NULL) +
    coord_flip() +
    theme_light() +
    theme(
      axis.title.x = element_blank(),
      axis.title.y = element_blank(),
      axis.ticks = element_blank(),
      panel.grid.major.y = element_blank(),
      panel.border = element_blank(),
      legend.title = element_blank(),
      legend.position = "none",
    ) +
    facet_grid(cols = vars(`Métrique`), scales = "free")

ggsave("repartition-generale.svg", width=10, height=6)

horde %>%
  mutate(Chapitre = fct_rev(fct_reorder(paste("Chapitre", Chapitre), Chapitre))) %>%
  filter(Partie %in% personnages) %>%
  mutate(Personnage = fct_lump_min(Partie, 9000, w=Signes, other_level="Autres")) %>%
  mutate(Personnage = fct_rev(fct_relevel(Personnage, personnages))) %>%
  mutate(Personnage = fct_relevel(Personnage, "Autres")) %>%
  ggplot(aes(x = Chapitre, y = Signes, fill = Personnage)) +
    geom_col(position = "fill") +
    scale_fill_personnages +
    scale_y_continuous(breaks = NULL) +
    coord_flip() +
    theme_light() +
    theme(
      axis.title.x = element_blank(),
      axis.title.y = element_blank(),
      axis.ticks = element_blank(),
      axis.text.x = element_blank(),
      axis.text.y = element_text(size = 16),
      panel.grid.major.y = element_blank(),
      panel.grid.major.x = element_blank(),
      panel.border = element_blank(),
      legend.key.size = unit(2, "line"),
    ) +
  guides(fill = guide_legend(reverse = TRUE))

ggsave("repartition-relative-par-chapitre.svg", width=10, height=10)

horde %>%
  mutate(Chapitre = fct_reorder(paste("Chapitre", Chapitre), Chapitre)) %>%
  filter(Partie %in% personnages) %>%
  transmute(Chapitre,
	    Personnage = Partie,
	    Signes) %>%
  mutate(Personnage = fct_rev(fct_relevel(Personnage, personnages))) %>%
  mutate(Personnage = fct_lump_min(Personnage, 9000, w=Signes, other_level="Autres")) %>%
  mutate(Personnage = fct_relevel(Personnage, "Autres")) %>%
  ggplot(aes(x = Personnage, y = Signes, fill = Personnage)) +
    geom_col(aes(fill = Personnage)) +
    scale_fill_personnages +
    scale_y_continuous(
      label = scales::label_number(),
      breaks = function(xs) { l <- ceiling(max(abs(xs))/10000)*10000
                              seq(-l, l, by=20000) },
    ) +
    coord_flip() +
    theme_light() +
    theme(
      axis.title.x = element_blank(),
      axis.title.y = element_blank(),
      axis.ticks = element_blank(),
      axis.text.y = element_blank(),
      panel.grid.major.y = element_blank(),
      panel.border = element_blank(),
      legend.key.size = unit(2, "line"),
    ) +
  facet_wrap(vars(Chapitre), ncol = 4) +
  guides(fill = guide_legend(reverse = TRUE))

ggsave("repartition-absolue-par-chapitre.svg", width=10, height=10)

horde %>%
  mutate(Chapitre = fct_rev(fct_reorder(paste("Chapitre", Chapitre), Chapitre))) %>%
  mutate(Signes_Personnages_Masculins = Signes * ifelse(Partie %in% personnages_masculins, 1, 0),
         Signes_Personnages_Feminins = Signes * ifelse(Partie %in% personnages_feminins, 1, 0)) %>%
  group_by(Chapitre) %>%
  summarize(Signes_Personnages_Masculins = sum(Signes_Personnages_Masculins),
            Signes_Personnages_Feminins = sum(Signes_Personnages_Feminins)) %>%
  ungroup() %>%
  ggplot(aes(x = Chapitre)) +
    geom_col(aes(y = -Signes_Personnages_Masculins, fill = "masculine")) +
    geom_label(aes(x="Chapitre 2", y=-70000, label="Personnages\nmasculins")) +
    geom_col(aes(y = Signes_Personnages_Feminins, fill = "féminine")) +
    geom_label(aes(x="Chapitre 2", y=70000, label="Personnages\nféminins")) +
    scale_fill_manual(
      values = c(masculine="#FF3030",féminine="#2C5372"),
    ) +
    scale_y_continuous(
      name = "Signes",
      label = scales::label_number(),
      limits = function(xs) c(-max(abs(xs)), max(abs(xs))),
      breaks = function(xs) { l <- ceiling(max(abs(xs))/10000)*10000
                              seq(-l, l, by=30000) },
    ) +
    coord_flip() +
    theme_light() +
    theme(
      text = element_text(size = 16),
      axis.title.y = element_blank(),
      axis.ticks = element_blank(),
      panel.grid.major.y = element_blank(),
      panel.border = element_blank(),
      legend.title = element_blank(),
      legend.position = "none",
    ) +
  guides(fill = guide_legend(reverse = TRUE))

ggsave("repartition-des-genres-par-chapitre.svg", width=10, height=10)

horde %>%
  mutate(Chapitre = fct_rev(fct_reorder(paste("Chapitre", Chapitre), Chapitre))) %>%
  transmute(Chapitre,
            Déséquilibre = Signes *
              case_when(Partie %in% personnages_masculins ~ -1,
                        Partie %in% personnages_feminins ~ 1,
                        TRUE ~ 0)) %>%
  group_by(Chapitre) %>%
  summarize(Déséquilibre = sum(Déséquilibre)) %>%
  ungroup() %>%
  mutate(Dominante = ifelse(Déséquilibre < 0, "masculine","féminine")) %>%
  ggplot(aes(x = Chapitre, y = Déséquilibre)) +
  geom_col(aes(fill = Dominante)) +
  scale_fill_manual(
    values = c(masculine="#FF3030",féminine="#2C5372"),
  ) +
  scale_y_continuous(
    name = "Différence de signes",
    label = scales::label_number(),
    limits = function(xs) c(-max(abs(xs)), max(abs(xs))),
    breaks = function(xs) { l <- ceiling(max(abs(xs))/10000)*10000
                            seq(-l, l, by=30000) },
  ) +
  coord_flip() +
  theme_light() +
  theme(
    text = element_text(size = 16),
    axis.title.y = element_blank(),
    axis.ticks = element_blank(),
    panel.grid.major.y = element_blank(),
    panel.border = element_blank(),
    legend.position=c(0.80, 0.80)
  ) +
  guides(fill = guide_legend(reverse = TRUE))

ggsave("dominante-de-genre-par-chapitre.svg", width=10, height=10)





par_chapitres <- horde %>%
  group_by(Chapitre) %>%
  mutate(Depart = cumsum(Signes) - Signes) %>%
  ungroup()

parties_plot <- function(df,
                         extra_theme = theme(),
                         extra_guides = guides()) {
  mutate(df,
         Partie = fct_rev(fct_relevel(Partie, patterns$Partie))) %>%
  mutate(Chapitre = fct_rev(fct_reorder(paste("Chapitre", Chapitre), Chapitre))) %>%
  ggplot(aes(x = Chapitre, y = Signes, fill = Partie)) +
    geom_col(position = position_nudge(y = df$Depart),
             colour = "lightblue",
             size = 0.1) +
    scale_fill_personnages +
    scale_x_discrete(name = "Chapitre") +
    scale_y_continuous(breaks = NULL) +
    coord_flip() +
    theme_light() +
    theme(
      axis.title.x = element_blank(),
      axis.title.y = element_blank(),
      axis.ticks = element_blank(),
      axis.text.y = element_text(size = 32),
      axis.text.x = element_blank(),
      panel.grid.major.y = element_blank(),
      panel.grid.major.x = element_blank(),
      panel.border = element_blank(),
      legend.text = element_text(size = 32),
      legend.title = element_blank(),
      legend.key.size = unit(5, "line"),
    ) +
    extra_theme +
    guides(fill = guide_legend(reverse = TRUE)) +
    extra_guides
}

svgout(filename = "cartographie.svg",
       width = 40,
       height = 20)
parties_plot(par_chapitres)
invisible(dev.off())

for (chap in sort(unique(par_chapitres$Chapitre))) {
  svgout(filename = paste0("cartographie-", sprintf("%02d", chap), ".svg"),
         width = 40,
         height = 3)

  par_chapitres %>%
    filter(Chapitre == chap) %>%
    parties_plot(theme(axis.text.y = element_blank(),
                       legend.position = "top",
                       legend.direction = "horizontal",
                       panel.grid = element_blank(),
                       panel.border = element_blank(),
                       panel.background = element_rect(fill = "transparent", colour = NA),
                       plot.background = element_rect(fill = "transparent", colour = NA),
                       panel.spacing = unit(c(0, 0, 0, 0), "null"),
                       plot.margin = unit(c(0, 0, 0, 0), "null"),
                       legend.key.size = unit(4, "line"),
                      ),
                 guides(fill = guide_legend(nrow=ifelse(chap == 10, 2, 1), reverse=TRUE))
                ) %>%
    grid.draw()
  invisible(dev.off())
}

